/*
GRN algorithm in an artificial evolution framework to model 
simple differentiation in two cells with the same genome
Copyright (C) 2005  Johannes Knabe
j.f.knabe@herts.ac.uk

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
public class twoCellEvo implements evo{

public static int popSize=500;
public static int compGroupSize=15;
public static int lifetime=500;
public static double toBeat=9999.0;//this feature might be used to speed evolution up a little:
                                   //all individuals worse who get worse than this value during their
                                   //lifetime are immediately terminated.
public static boolean useToBeat=false;//turn this of to assure that every individual is fully evaluated
public individual base;

/*this is the current population*/
public individual[] currentPop;


/*returns the current population*/
public individual[] getPopulation(){
return currentPop;
}

/*returns howMany randomly drawn individuals of current population*/
/*NB: an indivdual might be included several times!*/
public individual[] getPartPopulation(int howMany){
individual[] ret=new individual[howMany];
java.util.Random rand=new java.util.Random();
for(int i=0;i<howMany;i++){
  ret[i]=currentPop[rand.nextInt(currentPop.length)];
}
return ret;
}

public individual getFittest(individual[] pop){
double fit=9999.0;
int pos=-1;
for(int i=0;i<pop.length;i++)
  if((pop[i]!=null)&&(pop[i].getFitness()<fit)){
    fit=pop[i].getFitness();
    pos=i;
    }
if(pos>=0){
  if(useToBeat)
    toBeat=pop[pos].getFitness()+1.0;
  return pop[pos];
}
else
  return null;
}

public individual get2ndFittest(individual[] pop){
individual fit=getFittest(pop);
individual[] temp=new individual[pop.length-1];
for(int i=0;i<pop.length-1;i++)
  if(pop[i]!=fit) /*omit fittest, add all others*/
    temp[i]=pop[i];
return getFittest(temp);
}

/*returns fittest individual of current population*/
public individual getFittest(){
return getFittest(currentPop);
}

/*constructor: generate first generation and set some global params*/
public twoCellEvo(individual base){
currentPop=new individual[popSize];
this.base=base;
for(int i=0;i<popSize;i++)
  currentPop[i]=base.getRandomInd();
} 

public void processGen(){
processGen(999);
}

/*Run all individuals of current pop and assign fitness values*/
public void processGen(int gen){
fitness fit;
for(int i=0;i<popSize;i++){
//  if(gen<10)
//    fit=new cheapFit(20);
//  else
  fit=currentPop[i].getFit();
  int x=10;
  if(main.slowShift)
   if(gen<250)
    x=(int)(gen*1.0/25.0);
  fit.setCount(x);
  currentPop[i].live(currentPop[i].getIn(),fit,currentPop[i].getLifetime());
  //System.out.print(currentPop[i].getFitness()+" ");
  }
}

/*create next Generation from current population*/
public void nextGen(){
int count=popSize;
individual[] nextPop=new individual[popSize];
//begin elitism
individual[] pop=new individual[popSize];
for(int i=0;i<pop.length;i++)
  pop[i]=currentPop[i];
for(int i=0;i<base.getElitism();i++){
  individual fit=getFittest(pop);
  nextPop[--count]=new twoCellGRN(fit.getGenome());
  for(int j=0;j<pop.length;j++)
    if(pop[j]==fit) //kick this guy out!
      pop[j]=null;
}
//end elitism
individual[] parents=new individual[2];
individual[] offspring;
while(count>0){
  individual[] maybeParents=getPartPopulation(base.getCompPool());
  parents[0]=getFittest(maybeParents);
  parents[1]=get2ndFittest(maybeParents);
  offspring=base.getOffspring(parents);
  nextPop[--count]=offspring[0];
  if(count>0)
    nextPop[--count]=offspring[1];
  }
currentPop=nextPop;
}

} 
